/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.StatusTyp;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Helfer Klasse zum Zugriff auf die FG6-De
 * 
 * @author Pittner
 * 
 */
public class Fg6Util
{
  private static Collection<String> _fg6De = null;

  private static Map<String, String> _atgDe = new HashMap<String, String>();

  private static Map<String, StatusTyp> _statusDe = new HashMap<String, StatusTyp>();

  private static String _pid;

  public enum FG6_STATUS {
    OK, DISTURBED, FAILURE, UNDEFINED;
    public Image getImage()
    {
      return ImageFactory.getImage("icons/fg6_status_" + toString().toLowerCase() + ".png");
    }
  };

  /**
   * Relevante Attributgruppen fr die FG6
   */
  public final static String[] FG6_ATG = { "atg.tlsVltTrKontakt", "atg.tlsVltStromVersorung",
      "atg.tlsVltberspannungsSchutz", "atg.tlsVltHeizung", "atg.tlsVltLicht", "atg.tlsVltTemperaturberwachung",
      "atg.tlsVltDiebstahlVandalismusSchutz", "atg.tlsVltLftung", };

  public final static String FG6_ASP = "asp.tlsAntwort";

  /**
   * Liefert eine Liste der EAK eines SM, die DE der FG6 enthalten. EAK > 253 werden ausgefiltert.
   * 
   * @param connection
   *          DaV-Verbindung
   * @param pidSm
   *          Pid des Steuermoduls
   * @return Liste der FG6-EAK
   */
  public static List<String> getFg6Eak(ClientDavInterface connection, String pidSm)
  {

    List<String> listFg6Eak = new ArrayList<String>();
    DataModel dataModel = connection.getDataModel();
    SystemObject soSm = dataModel.getObject(pidSm);
    if (soSm != null && soSm.isOfType("typ.steuerModul"))
    {
      ConfigurationObject sm = (ConfigurationObject)soSm;
      NonMutableSet mengeEak = sm.getNonMutableSet("Eak");
      if (mengeEak != null)
      {
        List<SystemObject> listeEak = mengeEak.getElements();
        for (SystemObject soEak : listeEak)
        {
          if (soEak != null && soEak.isOfType("typ.eak"))
          {
            ConfigurationObject eak = (ConfigurationObject)soEak;
            AttributeGroup eakAtg = dataModel.getAttributeGroup("atg.eak");
            if (eakAtg != null)
            {
              Data eakData = eak.getConfigurationData(eakAtg);
              if (eakData != null)
              {
                int osi2Adr = eakData.getItem("OSI2Adresse").asUnscaledValue().intValue();
                if (osi2Adr < 254)
                {
                  NonMutableSet mengeDe = eak.getNonMutableSet("De");
                  if (mengeDe != null)
                  {
                    List<SystemObject> listeDe = mengeDe.getElements();
                    for (SystemObject soDe : listeDe)
                    {
                      if (soDe != null && soDe.isOfType("typ.deVlt"))
                      {
                        AttributeGroup deAtg = dataModel.getAttributeGroup("atg.de");
                        if (deAtg != null)
                        {
                          Data deData = soDe.getConfigurationData(deAtg);
                          if (deData != null)
                          {
                            int deKanal = deData.getItem("DEKanal").asUnscaledValue().intValue();
                            if (deKanal < 255)
                            {
                              listFg6Eak.add(soEak.getPid());
                              break;
                            }
                          }
                        }
                        else
                        {
                          System.err.println("Attributgruppe atg.de nicht vorhanden");
                          break;
                        }
                      }
                    }
                  }
                }
              }

            }
            else
            {
              System.err.println("Attributgruppe atg.eak nicht vorhanden");
              break;
            }
          }
        }
      }
    }
    return listFg6Eak;
  }

  /**
   * Liefert eine Liste der FG6-DE (typ.deVlt) eines EAK fr ein SM. DE > 254 werden ausgefiltert.
   * 
   * @param connection
   *          DaV-Verbindung
   * @param pidSm
   *          Pid des Steuermoduls
   * @param pidEak
   *          Pid des EAK
   * @return Liste der FG6-DE fr die bergebene EAK-Pid
   */
  public static List<String> getFg6De(ClientDavInterface connection, String pidSm, String pidEak)
  {
    List<String> listFg6De = new ArrayList<String>();
    DataModel dataModel = connection.getDataModel();
    SystemObject soEak = dataModel.getObject(pidEak);
    if (soEak != null && soEak.isOfType("typ.eak"))
    {
      ConfigurationObject eak = (ConfigurationObject)soEak;
      NonMutableSet mengeDe = eak.getNonMutableSet("De");
      if (mengeDe != null)
      {
        List<SystemObject> listeDe = mengeDe.getElements();
        for (SystemObject soDe : listeDe)
        {
          if (soDe != null && soDe.isOfType("typ.deVlt"))
          {
            AttributeGroup deAtg = dataModel.getAttributeGroup("atg.de");
            if (deAtg != null)
            {
              Data deData = soDe.getConfigurationData(deAtg);
              if (deData != null)
              {
                int deKanal = deData.getItem("DEKanal").asUnscaledValue().intValue();
                if (deKanal < 255)
                {
                  listFg6De.add(soDe.getPid());
                }
              }
            }
            else
            {
              System.err.println("Attributgruppe atg.de nicht vorhanden");
              break;
            }
          }
        }
      }
    }
    return listFg6De;
  }

  /**
   * Liefert eine Liste aller FG6-DE (typ.deVlt) einer SM (fr alle EAK der SM). DE > 254 werden ausgefiltert.
   * 
   * @param connection
   *          DaV-Verbindung
   * @param pidSm
   *          Pid des Steuermoduls
   * @return Liste der FG6-DE fr die bergebene SM-Pid
   */
  public static List<String> getAllFg6De(ClientDavInterface connection, String pidSm)
  {

    List<String> listFg6De = new ArrayList<String>();
    for (String pidEak : getFg6Eak(connection, pidSm))
    {
      listFg6De.addAll(getFg6De(connection, pidSm, pidEak));
    }
    return listFg6De;
  }

  /**
   * Bestimmt den FG6-Status eines Datensatzes
   * 
   * @param data
   *          der Datensatz, dessen Status bestimmt werden soll
   * 
   * @return den FG6-Status des Datensatzes
   */
  public static FG6_STATUS getFg6Status(String atgPid, Data data)
  {

    FG6_STATUS result = FG6_STATUS.UNDEFINED;

    if (atgPid.equals("atg.tlsVltTrKontakt"))
    {
      String tuerKontakt = data.getItem("Trkontakt").asTextValue().getText();
      if (tuerKontakt.equals("Tr geschlossen"))
      {
        result = FG6_STATUS.OK;
      }
      else if (tuerKontakt.equals("Tr offen"))
      {
        result = FG6_STATUS.DISTURBED;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + tuerKontakt + "\" fr Attribut Trkontakt der ATG " + atgPid
            + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltStromVersorung"))
    {
      String netzSpannung = data.getItem("Netzspannung").asTextValue().getText();
      if (netzSpannung.equals("Netzspannung ok"))
      {
        result = FG6_STATUS.OK;
      }
      else if (netzSpannung.equals("Netzspannung ausgefallen"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + netzSpannung + "\" fr Attribut Netzspannung der ATG " + atgPid
            + " wird IGNORIERT");
      }
      String usv = data.getItem("USV").asTextValue().getText();
      if (usv.equals("USV ok"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (usv.equals("USV defekt"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + usv + "\" fr Attribut USV der ATG " + atgPid + " wird IGNORIERT");
      }
      String akkuEntladung = data.getItem("Akkuentladung").asTextValue().getText();
      if (akkuEntladung.equals("Akku nicht entladen"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (akkuEntladung.equals("Akku entladen"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + akkuEntladung + "\" fr Attribut Akkuentladung der ATG " + atgPid
            + " wird IGNORIERT");
      }
      String akkuUeberladung = data.getItem("Akkuberladung").asTextValue().getText();
      if (akkuUeberladung.equals("Akku nicht berladen"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (akkuUeberladung.equals("Akku berladen"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + akkuUeberladung + "\" fr Attribut Akkuberladung der ATG " + atgPid
            + " wird IGNORIERT");
      }
      String fiSchutzSchalter = data.getItem("FISchutzschalter").asTextValue().getText();
      if (fiSchutzSchalter.equals("FI-Schutzschalter eingeschaltet"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (fiSchutzSchalter.equals("FI-Schutzschalter ausgelst"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + fiSchutzSchalter + "\" fr Attribut FISchutzschalter der ATG "
            + atgPid + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltberspannungsSchutz"))
    {
      String ueberSpannungsSchutz = data.getItem("berspannungsschutz").asTextValue().getText();
      if (ueberSpannungsSchutz.equals("berspannungsschutz ok"))
      {
        result = FG6_STATUS.OK;
      }
      else if (ueberSpannungsSchutz.equals("berspannungsschutz defekt"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + ueberSpannungsSchutz
            + "\" fr Attribut berspannungsschutz der ATG " + atgPid + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltHeizung"))
    {
      String heizungStatus = data.getItem("HeizungStatus").asTextValue().getText();
      if (heizungStatus.equals("Heizung ok"))
      {
        result = FG6_STATUS.OK;
      }
      else if (heizungStatus.equals("Heizung defekt"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + heizungStatus + "\" fr Attribut HeizungStatus der ATG " + atgPid
            + " wird IGNORIERT");
      }
      // HeizungBetriebsZustand trgt nicht zum Status bei
    }
    else if (atgPid.equals("atg.tlsVltLicht"))
    {
      String licht = data.getItem("Licht").asTextValue().getText();
      if (licht.equals("Licht eingeschaltet"))
      {
        result = FG6_STATUS.OK;
      }
      else if (licht.equals("Licht ausgeschaltet"))
      {
        // result = FG6_STATUS.DISTURBED;
        // Bei ausgeschaltetem Licht soll Status nicht auf "gelb" gehen
        result = FG6_STATUS.OK;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + licht + "\" fr Attribut Licht der ATG " + atgPid
            + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltTemperaturberwachung"))
    {
      String temperaturberwachung = data.getItem("Temperaturberwachung").asTextValue().getText();
      if (temperaturberwachung.equals("Temperaturberwachung ok"))
      {
        result = FG6_STATUS.OK;
      }
      else if (temperaturberwachung.equals("Temperaturberwachung defekt"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + temperaturberwachung
            + "\" fr Attribut Temperaturberwachung der ATG " + atgPid + " wird IGNORIERT");
      }
      String temperaturBereichObereGrenze = data.getItem("TemperaturBereichObereGrenze").asTextValue().getText();
      if (temperaturBereichObereGrenze.equals("Temperaturbereich nicht berschritten"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (temperaturBereichObereGrenze.equals("Temperaturbereich berschritten"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + temperaturBereichObereGrenze
            + "\" fr Attribut TemperaturBereichObereGrenze der ATG " + atgPid + " wird IGNORIERT");
      }
      String temperaturBereichUntereGrenze = data.getItem("TemperaturBereichUntereGrenze").asTextValue().getText();
      if (temperaturBereichUntereGrenze.equals("Temperaturbereich nicht unterschritten"))
      {
        result = result == FG6_STATUS.FAILURE || result == FG6_STATUS.DISTURBED ? result : FG6_STATUS.OK;
      }
      else if (temperaturBereichUntereGrenze.equals("Temperaturbereich unterschritten"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + temperaturBereichUntereGrenze
            + "\" fr Attribut TemperaturBereichUntereGrenze der ATG " + atgPid + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltDiebstahlVandalismusSchutz"))
    {
      String diebstahl = data.getItem("Diebstahl").asTextValue().getText();
      if (diebstahl.equals("kein Alarm"))
      {
        result = FG6_STATUS.OK;
      }
      else if (diebstahl.equals("ALARM"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + diebstahl + "\" fr Attribut Diebstahl der ATG " + atgPid
            + " wird IGNORIERT");
      }
    }
    else if (atgPid.equals("atg.tlsVltLftung"))
    {
      String lueftungStatus = data.getItem("LftungStatus").asTextValue().getText();
      if (lueftungStatus.equals("Lftung ok"))
      {
        result = FG6_STATUS.OK;
      }
      else if (lueftungStatus.equals("Lftung defekt"))
      {
        result = FG6_STATUS.FAILURE;
      }
      else
      {
        System.err.println("Unbekannter Wert \"" + lueftungStatus + "\" fr Attribut LftungStatus der ATG " + atgPid
            + " wird IGNORIERT");
      }
      // LftungBetriebsZustand trgt nicht zum Status bei
    }
    else
    {
      System.err.println("Unerwartete Attributgruppe \"" + atgPid + "\"");
    }

    return result;
  }

  public static AnzeigeInhaltTyp ermittleStatusAusEmpfangsDaten()
  {
    StatusTyp statusNeu = null;

    /*
     * Feststellen hchster Status aller DE
     */

    for (String dePid : getFg6De())
    {
      StatusTyp status = _statusDe.get(dePid);

      if (statusNeu == null)
      {
        statusNeu = status;
      }
      else if (status == StatusTyp.GESTOERT)
      {
        statusNeu = status;
      }
      else if (status == StatusTyp.KOMMUNIKATIONSAUSFALL)
      {
        if (statusNeu != StatusTyp.GESTOERT)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.TEILSTOERUNG)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.TUERKONTAKT)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL
            && statusNeu != StatusTyp.TEILSTOERUNG)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.OK)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL
            && statusNeu != StatusTyp.TEILSTOERUNG && statusNeu != StatusTyp.TUERKONTAKT)
        {
          statusNeu = status;
        }
      }
    }

    /*
     * Umsetzen Status in Anzeigeinhalt
     */

    AnzeigeInhaltTyp result = AnzeigeInhaltTyp.UNDEFINIERT;

    if (statusNeu == StatusTyp.GESTOERT || statusNeu == StatusTyp.KOMMUNIKATIONSAUSFALL)
    {
      result = AnzeigeInhaltTyp.GESTOERT;
    }
    if (statusNeu == StatusTyp.TEILSTOERUNG || statusNeu == StatusTyp.TUERKONTAKT)
    {
      result = AnzeigeInhaltTyp.TEILSTOERUNG;
    }
    if (statusNeu == StatusTyp.OK)
    {
      result = AnzeigeInhaltTyp.OK;
    }
    return result;
  }

  /**
   * @return
   */
  public static Collection<String> getFg6De()
  {
    if (_fg6De == null)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        _fg6De = new HashSet<String>();
        _fg6De.addAll(Fg6Util.getAllFg6De(connection, getPid()));
        return Collections.unmodifiableCollection(_fg6De);
      }
      else
      {
        return Collections.emptyList();
      }
    }
    else
    {
      return Collections.unmodifiableCollection(_fg6De);
    }
  }

  public static void setResultData(String pidSS, ResultData result)
  {
    _pid = pidSS;
    _fg6De = null;

    SystemObject so = result.getObject();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    String atgDe = _atgDe.get(so.getPid());
    Data data = result.getData();

    if (data != null)
    {

      // System.out.println("FG6-Daten empfangen fr SSt " + getPid() +
      // "\n\t DE " + so.getPid() +
      // "\n\t " + result.getData());

      if (atgDe != null)
      {
        if (atgDe.equals(atg.getPid()))
        {
          // Es werden Daten derselbenATG empfangen wie vorher
          setzeStatus(so.getPid(), atg.getPid(), data);
        }
        else
        {
          // Es werden Daten einer ANDEREN ATG empfangen
          _atgDe.put(so.getPid(), atg.getPid());
          setzeStatus(so.getPid(), atg.getPid(), data);
        }
      }
      else
      {
        // Es werden Daten empfangen, vorher war "keine Daten"
        _atgDe.put(so.getPid(), atg.getPid());
        setzeStatus(so.getPid(), atg.getPid(), data);
      }
    }
    else
    {
      if (atgDe != null)
      {
        if (atgDe.equals(atg.getPid()))
        {
          // Vorher empfangene ATG geht auf "keine Daten"
          _atgDe.remove(so.getPid());
        }
      }
    }

    /*
     * Prfen, ob fr alle DE Daten vorliegen, ansonsten liegt Kommunikationsstrung vor
     */
    for (String dePid : getFg6De())
    {
      if (_atgDe.get(dePid) == null)
      {
        _statusDe.put(dePid, StatusTyp.KOMMUNIKATIONSAUSFALL);
      }
    }
  }

  /**
   * Holt die Pid des Steuermoduls
   * 
   * @return die Pid
   */
  public static String getPid()
  {
    return _pid;
  }

  /**
   * Setzt die Pid des Steuermoduls
   */
  public void setPid(String pid)
  {
    _pid = pid;
  }

  private static void setzeStatus(String soPid, String atgPid, Data data)
  {
    FG6_STATUS status = Fg6Util.getFg6Status(atgPid, data);

    switch (status)
    {
      case OK:
        _statusDe.put(soPid, StatusTyp.OK);
        break;

      case DISTURBED:
        if (atgPid.equals("atg.tlsVltTrKontakt"))
          _statusDe.put(soPid, StatusTyp.TUERKONTAKT);
        else
          _statusDe.put(soPid, StatusTyp.TEILSTOERUNG);
        break;

      case FAILURE:
        _statusDe.put(soPid, StatusTyp.GESTOERT);
        break;

      case UNDEFINED:
        break; // ignore
    }

  }

}
