/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.pua;

import java.util.Vector;

import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.pua.prot.client.ProtocolListener;
import de.bsvrz.pua.prot.client.ProtocolResultStream;
import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.losb.datk.AtlMeta;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

/**
 * Klasse realisiert den synchronisierten Zugriff auf die PUA, die Klasse wird als Singleton ausgelegt
 * 
 * @author Dambach Werke GmbH
 */
public class ZugriffPua
{
  private final boolean _debug = false;

  /**
   * Speichert die Singelton Instanz der Klasse
   */
  private static ZugriffPua _instanz = null;

  /**
   * Instanz PUA
   */
  private ProtokolleUndAuswertungen m_pua = null;

  /**
   * Privater Konstruktor
   * 
   * @param connection
   */
  private ZugriffPua(ClientDavInterface connection)
  {
    m_pua = new ProtokolleUndAuswertungen(connection);
  }

  /**
   * Liefert die Singleton Instanz der Klasse {@link ZugriffPua}
   * 
   * @param connection
   *          Verbindung zum DaV
   * @return Singleton Instanz der Klasse {@link ZugriffPua}
   */
  public static ZugriffPua getInstanz(ClientDavInterface connection)
  {
    if (_instanz == null)
    {
      synchronized (ZugriffPua.class)
      {
        if (_instanz == null)
        {
          _instanz = new ZugriffPua(connection);
        }
      }
    }

    return _instanz;
  }

  public ProtocolRequestResult loescheGespeichertesProtokoll(long id)
  {
    ProtocolRequestResult r;

    synchronized (ZugriffPua.class)
    {
      r = m_pua.loescheGespeichertesProtokoll(id);
    }
    return r;
  }

  public Vector<ProtokollInformationen> getListeUngeleseneProtokolle()
  {
    Vector<ProtokollInformationen> r;

    synchronized (ZugriffPua.class)
    {
      r = m_pua.getListeUngeleseneProtokolle();
    }
    return r;
  }

  public Vector<ProtokollInformationen> getListeGespeicherteProtokolle()
  {
    Vector<ProtokollInformationen> r;

    synchronized (ZugriffPua.class)
    {
      r = m_pua.getListeGespeicherteProtokolle();
    }

    return r;
  }

  public boolean aktualisiereListeUngeleseneProtokolle(String benutzer)
  {
    boolean r;

    synchronized (ZugriffPua.class)
    {
      r = m_pua.aktualisiereListeUngeleseneProtokolle(benutzer);
    }

    return r;
  }

  public boolean aktualisiereListeGespeicherteProtokolle(String benutzer)
  {
    boolean r;

    synchronized (ZugriffPua.class)
    {
      if (_debug)
        System.out.println("Start aktualisiereListeGespeicherteProtokolle");
      r = m_pua.aktualisiereListeGespeicherteProtokolle(benutzer);
      if (_debug)
        System.out.println("Beende aktualisiereListeGespeicherteProtokolle");
    }

    return r;
  }

  public ProtocolResultStream zugriffAufUngelesenesProtokoll(long id)
  {
    ProtocolResultStream stream;

    synchronized (ZugriffPua.class)
    {
      stream = m_pua.zugriffAufUngelesenesProtokoll(id);
    }

    return stream;
  }

  public ProtocolResultStream zugriffAufGespeichertesProtokoll(long id)
  {
    ProtocolResultStream stream;

    synchronized (ZugriffPua.class)
    {
      stream = m_pua.zugriffAufGespeichertesProtokoll(id);
    }

    return stream;
  }

  public void printStream(ProtocolResultStream stream, ProtokollInformationen info, boolean ausgabeAlsFrame)
  {
    synchronized (ZugriffPua.class)
    {
      if (_debug)
        System.out.println("Starte printStream");
      m_pua.printStream(stream, info, ausgabeAlsFrame);
      if (_debug)
        System.out.println("Beende printStream");
    }
  }

  public ProtocolResultStream erstelleProtokollMitSofortigerAusgabe(ProcessingParameter processingParameter)
      throws FailureException
  {
    synchronized (ZugriffPua.class)
    {
      return m_pua.erstelleProtokollMitSofortigerAusgabe(processingParameter);
    }
  }

  public ProtocolResultStream erstelleProtokollOhneAusgabe(ProcessingParameter processingParameter)
      throws FailureException
  {
    synchronized (ZugriffPua.class)
    {
      return m_pua.erstelleProtokollOhneAusgabe(processingParameter);
    }
  }

  public ProtocolRequestResult abbruchProtokollerstellung(long id)
  {
    synchronized (ZugriffPua.class)
    {
      return m_pua.abbruchProtokollerstellung(id);
    }
  }

  public ProtocolRequestResult holeSkriptInformationen(DynamicObject skript, StringBuffer buffer, AtlMeta meta)
  {
    synchronized (ZugriffPua.class)
    {
      if (_debug)
        System.out.println("Beginn holeSkriptInformationen");
      ProtocolRequestResult r = m_pua.holeSkriptInformationen(skript, buffer, meta);
      if (_debug)
        System.out.println("Ende holeSkriptInformationen");

      return r;
    }
  }

  /**
   * Fuegt einen ProtocolListener fuer Protokoll-Fertigstellungen hinzu.
   * 
   * @param listener ProtocolListener fuer Protokoll-Fertigstellungen
   *          
   * @return ProtocolRequestResult Ergebnis der Listener-Registrierung
   * @throws FailureException
   *           Ausnahme bei der Anmeldung der Empfaengerklasse.
   */
  public ProtocolRequestResult addProtokollListener(final ProtocolListener listener) throws FailureException
  {
    return m_pua.addProtokollListener(listener);
  }

  /**
   * Entfernt einen ProtocolListener fuer Protokoll-Fertigstellungen.
   * 
   * @param listener ProtocolListener fuer Protokoll-Fertigstellungen
   *          
   * @return ProtocolRequestResult Ergebnis der Listener-Deregistrierung
   * @throws FailureException
   *           Ausnahme bei der Abmeldung der Empfaengerklasse.
   */
  public ProtocolRequestResult removeProtokollListener(final ProtocolListener listener) throws FailureException
  {
    return m_pua.removeProtokollListener(listener);
  }
}
