/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

/**
 * Adapter fr das Managment einer ganzen Hierarchie von Objekten und deren
 * Kindern (z.B. Member oder auch Referenzen).
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: AbstractDependantAttributeAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
 */
public abstract class AbstractDependantAttributeAdapter extends AdapterImpl {

	/**
	 * Adapter fr das Management aller Attributnderungen eines Kindobjektes
	 * innherhalb einer Hierarchie von Objekten.
	 * 
	 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
	 * @version $Id: AbstractDependantAttributeAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
	 */
	private final class NavigationAdapter extends AdapterImpl {

		/**
		 * Liste mit allen {@link EStructuralFeature}s, auf die dieser Adapter
		 * reagiert.
		 */
		private final List<EStructuralFeature> dependantFeatures = new ArrayList<EStructuralFeature>();

		@Override
		public void notifyChanged(final Notification notification) {
			final int eventType = notification.getEventType();
			if ((Notification.SET == eventType || Notification.UNSET == eventType)
					&& dependantFeatures.contains(notification.getFeature())) {
				notifyNavigationTargetChange(notification);
			}
		}

		/**
		 * Ein Kindobjekt eines {@link EObject}s wurde hinzugefgt oder auch
		 * entfernt.
		 * 
		 * @param notification
		 *            die konkrete {@link Notification}.
		 */
		@SuppressWarnings("unchecked")
		public void notifyChangedNavigation(final Notification notification) {
			switch (notification.getEventType()) {
			case Notification.ADD:
				final EObject added = (EObject) notification.getNewValue();
				added.eAdapters().add(this);
				break;
			case Notification.ADD_MANY:
				final Collection<EObject> addedCol = (Collection<EObject>) notification
						.getNewValue();
				for (final EObject added2 : addedCol) {
					added2.eAdapters().add(this);
				}
				break;
			case Notification.SET:
			case Notification.UNSET:
				final EObject newValue = (EObject) notification.getNewValue();
				final EObject oldValue = (EObject) notification.getOldValue();
				if (oldValue != null) {
					oldValue.eAdapters().remove(this);
				}
				if (newValue != null) {
					newValue.eAdapters().add(this);
				}
				break;
			case Notification.REMOVE:
				final EObject removed = (EObject) notification.getOldValue();
				removed.eAdapters().remove(this);
				break;
			case Notification.REMOVE_MANY:
				final Collection<EObject> removedCol = (Collection<EObject>) notification
						.getOldValue();
				for (final EObject removed2 : removedCol) {
					removed2.eAdapters().remove(this);
				}
				break;
			default:
				// No notification
				return;
			}
			notifyDependantChange(notification);
		}
	}

	/**
	 * Das Objekt auf dessen Eigenschaftsnderungen der Adapter reagiert.
	 */
	private final InternalEObject source;

	/**
	 * Menge von Eigenschaften, die zum feuern des Adapters fhren.
	 */
	private final List<EStructuralFeature> localFeatures = new ArrayList<EStructuralFeature>();

	/**
	 * Map mit der sich die {@link EStructuralFeature}s und die dazugehrigen
	 * {@link NavigationAdapter} gemerkt werden.
	 */
	private final Map<EStructuralFeature, NavigationAdapter> navigationFeatureMap = new HashMap<EStructuralFeature, NavigationAdapter>();

	/**
	 * Erzeugt einen neuen {@link AbstractDependantAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public AbstractDependantAttributeAdapter(final EObject source,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature,
			final EStructuralFeature localFeature) {
		this(source);
		addNavigatedDependency(navigationFeature, dependantFeature);
		addLocalDependency(localFeature);
	}

	/**
	 * Erzeugt einen neuen {@link AbstractDependantAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 */
	public AbstractDependantAttributeAdapter(final EObject source,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature) {
		this(source);
		addNavigatedDependency(navigationFeature, dependantFeature);
	}

	/**
	 * Erzeugt einen neuen {@link AbstractDependantAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public AbstractDependantAttributeAdapter(final EObject source,
			final EStructuralFeature localFeature) {
		this(source);
		addLocalDependency(localFeature);
	}

	/**
	 * Erzeugt einen neuen {@link AbstractDependantAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 */
	public AbstractDependantAttributeAdapter(final EObject source) {
		super();
		this.source = (InternalEObject) source;
		source.eAdapters().add(this);
	}

	/**
	 * Registriert den {@link AbstractDependantAttributeAdapter} fr ein
	 * {@link EStructuralFeature}, das die Objekthierarchie beschreibt (als
	 * Kinder) und fr ein {@link EStructuralFeature}, das eine Eigenschaft
	 * eines Kindes beschreibt.
	 * 
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 */
	public void addNavigatedDependency(
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature) {
		NavigationAdapter navigationAdapter = navigationFeatureMap
				.get(navigationFeature);
		if (null == navigationAdapter) {
			navigationAdapter = new NavigationAdapter();
			navigationFeatureMap.put(navigationFeature, navigationAdapter);
		}
		navigationAdapter.dependantFeatures.add(dependantFeature);

	}

	/**
	 * Registriert den {@link AbstractDependantAttributeAdapter} fr ein
	 * {@link EStructuralFeature} an dem Wurzelelement der Objekthierarchie bzw.
	 * dem Quell - {@link EObject}, an dem der
	 * {@link AbstractDependantAttributeAdapter} registriert wurde.
	 * 
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public void addLocalDependency(final EStructuralFeature localFeature) {
		localFeatures.add(localFeature);
	}

	@Override
	public void notifyChanged(final Notification notification) {
		final NavigationAdapter navigationAdapter = navigationFeatureMap
				.get(notification.getFeature());
		if (null != navigationAdapter) {
			navigationAdapter.notifyChangedNavigation(notification);
		} else if (localFeatures.contains(notification.getFeature())) {
			notifyDependantChange(notification);
		}
	}

	/**
	 * Wird aufgerufen, wenn sich innerhalb der Objekthierarchie ein Attribut
	 * eines Kindes gendert hat.
	 * 
	 * @param notification
	 *            die {@link Notification}.
	 */
	protected abstract void notifyDependantChange(Notification notification);

	/**
	 * Wird aufgerufen, wenn sich die Struktur der Objekthierachie gendert hat.
	 * 
	 * @param notification
	 *            die {@link Notification}
	 */
	protected abstract void notifyNavigationTargetChange(
			Notification notification);

	/**
	 * Liefert das Wurzelobjekt der Objekthierachie, die durch den
	 * {@link AbstractDependantAttributeAdapter} berwacht wird.
	 * 
	 * @return das Wurzelobjekt der Objekthierachie.
	 */
	public InternalEObject getSource() {
		return source;
	}
}
