/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import de.bsvrz.dav.daf.main.Data;

/**
 * Kapselt alle notwendigen Daten zur eindeutigen Identifikation eines
 * Datensatzes.
 * 
 * @author HofmannJ
 * 
 */
public class ParameterEditorKapsel implements Comparable<String>
{
	/**
	 * Pid des SystemObjektes
	 */
	private String _pidSystemObjekt = null;

	/**
	 * Pid der Attributgruppe
	 */
	private String _pidAtg = null;

	/**
	 * Pid des Aspektes fr den Empfang
	 */
	private String _pidAspEmpfang = null;

	/**
	 * Pid des Aspekts zum Senden
	 */
	private String _pidAspSenden = null;

	/**
	 * Simulationsvariante
	 */
	private short _simVar = 0;

	/**
	 * Data Empfangen
	 */
	private Data _dataEmpfangen = null;

	/**
	 * Data Gesendet
	 */
	private Data _dataSenden = null;

	/**
	 * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
	 * Identifikation eines Datensatzes.
	 * 
	 * @param pidSystemObjekt
	 *          Pid des SystemObjektes
	 * @param pidAtg
	 *          Pid der Attributgruppe
	 * @param pidAspEmpfang
	 *          Pid des Aspektes zum Empfang
	 * @param pidAspSenden
	 *          Pid des Aspektes zum Senden
	 * @param simVar
	 *          Simulationsvariante
	 */
	public ParameterEditorKapsel(String pidSystemObjekt, String pidAtg,
			String pidAspEmpfang, String pidAspSenden, short simVar)
	{
		this._pidSystemObjekt = pidSystemObjekt;
		this._pidAtg = pidAtg;
		this._pidAspEmpfang = pidAspEmpfang;
		this._pidAspSenden = pidAspSenden;
		this._simVar = simVar;
	}

	/**
	 * Konstruktor. bernimmt die notwendigen Parameter zur eindeutigen
	 * Identifikation eines Datensatzes. <br>
	 * Als Aspekt fr die Empfangsanmeldung wird <code>asp.parameterSoll</code>
	 * verwendet. Als Aspekt fr die Sendeanmeldung wird
	 * <code>asp.parameterVorgabe</code> verwendet.
	 * 
	 * @param pidSystemObjekt
	 *          Pid des SystemObjektes
	 * @param pidAtg
	 *          Pid der Attributgruppe
	 * @param simVar
	 *          Simulationsvariante
	 */
	public ParameterEditorKapsel(String pidSystemObjekt, String pidAtg,
			short simVar)
	{
		this._pidSystemObjekt = pidSystemObjekt;
		this._pidAtg = pidAtg;
		this._pidAspEmpfang = "asp.parameterSoll";
		this._pidAspSenden = "asp.parameterVorgabe";
		this._simVar = simVar;
	}

	/**
	 * Liefert die Pid des SystemObjektes.
	 * 
	 * @return Pid des SystemObjektes
	 */
	public String getPidSystemObjekt()
	{
		return _pidSystemObjekt;
	}

	/**
	 * Liefert die Pid der Attributgruppe.
	 * 
	 * @return Pid der Attributgruppe
	 */
	public String getPidAtg()
	{
		return _pidAtg;
	}

	/**
	 * Liefert die Pid des Aspektes zum Empfangen.
	 * 
	 * @return Pid des Aspektes zum Empfangen
	 */
	public String getPidAspEmpfang()
	{
		return _pidAspEmpfang;
	}

	/**
	 * Liefert die Pid des Aspektes zum Senden.
	 * 
	 * @return Pid des Aspektes zum Senden
	 */
	public String getPidAspSenden()
	{
		return _pidAspSenden;
	}

	/**
	 * Liefert die Simulationsvariante.
	 * 
	 * @return Simulationsvariante
	 */
	public short getSimulationsVariante()
	{
		return _simVar;
	}

	/**
	 * Liefert das {@link Data}-Objekt.
	 * 
	 * @return {@link Data}-Objekt
	 */
	public Data getDataEmpfangen()
	{
		return _dataEmpfangen;
	}

	/**
	 * Setzt das {@link Data}-Objekt.
	 * 
	 * @param data
	 *          {@link Data}-Objekt
	 */
	public void setDataEmpfangen(Data data)
	{
		this._dataEmpfangen = data;
	}

	/**
	 * Liefert die Daten, die gesendet werden sollen.
	 * 
	 * @return Zu sendende Daten.
	 */
	public Data getDataSenden()
	{
		return this._dataSenden;
	}

	/**
	 * Setzt die Daten, die gesendet werden sollen.
	 * 
	 * @param dataSenden
	 *          Zu sendende Daten.
	 */
	public void setDataSenden(Data dataSenden)
	{
		this._dataSenden = dataSenden;
	}

	public String toString()
	{
		String re = "";

		re = re + "Pid SystemObjekt: " + _pidSystemObjekt + "\nPid ATG: " + _pidAtg
				+ "\nPid ASP Empfang: " + _pidAspEmpfang + "\nPid ASP Senden: "
				+ _pidAspSenden + "\nSimulationsvariante: " + _simVar;

		return re;
	}

	/**
	 * Liefert die ID dieser Kapsel bestehend aus
	 * <code>Pid Systemobjekt;Pid_ATG;Pid_ASP_Empfang;Pid_ASP_Senden;Simulationsvariante</code>
	 * 
	 * @return ID dieser Kapsel
	 */
	public String getId()
	{
		String id = "";

		id = id + _pidSystemObjekt + ",";
		id = id + _pidAtg + ",";
		id = id + _pidAspEmpfang + ",";
		id = id + _pidAspSenden + ",";
		id = id + _simVar;

		return id;
	}

	/**
	 * Baut aus der ID einer {@link ParameterEditorKapsel} eine neue
	 * {@link ParameterEditorKapsel} auf.
	 * 
	 * @param id
	 *          ID der {@link ParameterEditorKapsel}.
	 * @return Aus ID generierte {@link ParameterEditorKapsel}.
	 */
	public static ParameterEditorKapsel baueKapselAusId(String id)
	{
		ParameterEditorKapsel re = null;

		String[] split = id.split(",");

		if (split.length == 5)
		{
			String pidSystemObject = split[0];
			String pidAtg = split[1];
			String pidAspEmpfang = split[2];
			String pidAspSenden = split[3];
			String strSimVar = split[4];
			short simVar = 0;
			try
			{
				simVar = Short.valueOf(strSimVar);
			}
			catch (NumberFormatException nfe)
			{
				nfe.printStackTrace();
			}

			re = new ParameterEditorKapsel(pidSystemObject, pidAtg, pidAspEmpfang,
					pidAspSenden, simVar);
		}

		return re;
	}

	@Override
	public int compareTo(String o)
	{
		int re = 0;

		if (_pidSystemObjekt != null)
		{
			re = _pidSystemObjekt.compareTo(o);
		}

		return re;
	}

}
