/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Kommunikation mit ATG <code>atg.tsfAbschnitte</code>. <br>
 * <b>Lesen</b>
 * <ul>
 * <li>TYP: <code>typ.temporaereSeitenstreifenFreigabe</code></li>
 * <li>ATG: <code>atg.tsfAbschnitte</code></li>
 * <li>ASP: <code>asp.eigenschaften</code></li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFAbschnitte
{

	private static DaVKomTSFAbschnitte _instance = null;

	/**
	 * Verbindung DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Value = SystemObjekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>, Value = Reprsentation
	 * von atg.tsfAbschnitte.
	 */
	private HashMap<SystemObject, M_TSFAbschnitte> _map = new HashMap<SystemObject, M_TSFAbschnitte>();

	private DaVKomTSFAbschnitte(ClientDavInterface con)
	{
		if (con != null)
		{
			this._connection = con;
		}

		getDaten();
	}

	public static DaVKomTSFAbschnitte getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFAbschnitte.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFAbschnitte(null);
				}
			}
		}

		return _instance;
	}

	public static DaVKomTSFAbschnitte getInstance(ClientDavInterface con)
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFAbschnitte.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFAbschnitte(con);
				}
			}
		}

		return _instance;
	}

	/**
	 * Liefert die Verbindung zum DaV.
	 * 
	 * @return Verbindung zum DaV.
	 */
	private ClientDavInterface getConnection()
	{
		if (_connection == null)
		{
			this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		return _connection;
	}

	/**
	 * Liefert die Objekte der Abschnitts(-Bereiche) der TSF
	 * 
	 * @return Array mit den Objekten.
	 */
	public SystemObject[] getAbschnittsObjekte()
	{
		LinkedList<SystemObject> listeAbsObjs = new LinkedList<SystemObject>();

		if (getConnection() != null)
		{
			DataModel dataModel = getConnection().getDataModel();
			SystemObjectType sot = dataModel
					.getType(Defs.TYP_TEMP_SEITENSTREIFEN_FREIGABE);
			if (sot != null)
			{
				List<SystemObject> lSO = sot.getObjects();
				listeAbsObjs.addAll(lSO);
			}
		}

		return listeAbsObjs.toArray(new SystemObject[listeAbsObjs.size()]);
	}

	/**
	 * Liefert zu einer Pid vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code> entsprechend
	 * {@link M_TSFAbschnitte}.
	 * 
	 * @param pid
	 *          Pid vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return {@link M_TSFAbschnitte} oder <code>null</code>, wenn zur gegebenen
	 *         Pid nicht verfgbar.
	 */
	public M_TSFAbschnitte getMTSFabschnitte(String pid)
	{
		M_TSFAbschnitte re = null;

		if (getConnection() != null)
		{
			SystemObject so = getConnection().getDataModel().getObject(pid);
			if (so != null)
			{
				re = getMTSFabschnitte(so);
			}
		}

		return re;
	}

	/**
	 * Liefert zu einem Objekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code> entsprechend
	 * {@link M_TSFAbschnitte}.
	 * 
	 * @param so
	 *          Objekt vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return {@link M_TSFAbschnitte} oder <code>null</code>, wenn zum gegebenen
	 *         {@link SystemObject} nicht verfgbar.
	 */
	public M_TSFAbschnitte getMTSFabschnitte(SystemObject so)
	{
		M_TSFAbschnitte re = null;

		if (so != null)
		{
			synchronized (_map)
			{
				re = _map.get(so);
			}
		}

		return re;
	}

	private void getDaten()
	{
		if (getConnection() != null)
		{
			DataModel dataModel = getConnection().getDataModel();
			AttributeGroup ATG = dataModel.getAttributeGroup(Defs.ATG_TFS_ABSCHNITTE);
			Aspect ASP = dataModel.getAspect(Defs.ASP_EIGENSCHAFTEN);

			SystemObject[] aSO = getAbschnittsObjekte();
			for (SystemObject so : aSO)
			{
				if (so != null)
				{
					Data[] data = dataModel.getConfigurationData(
							new SystemObject[] { so }, ATG, ASP);
					if (data.length > 0)
					{
						M_TSFAbschnitte abs = new M_TSFAbschnitte(so, data[0]);
						synchronized (_map)
						{
							_map.put(so, abs);
						}

//						System.out.println(abs);
//						System.out.println();
					}
				}
			}
		}
	}

	protected static ClientDavConnection getDaVConnection()
	{
		ClientDavConnection con = null;
		String ip = "192.0.1.234";
		int port = 8083;
		String user = "Tester";
		String password = "geheim";

		try
		{
			con = new ClientDavConnection();
			ClientDavParameters par = con.getClientDavParameters();
			par.setDavCommunicationAddress(ip);
			par.setDavCommunicationSubAddress(port);
			par.setConfigurationPath("C:/temp/");
			try
			{
				con.connect();
				try
				{
					con.login(user, password);
					System.out.println("=>Verbindung aufgebaut, Benutzer angemeldet.");
				} catch (InconsistentLoginException e)
				{
					e.printStackTrace();
				}

			} catch (CommunicationError e)
			{
				e.printStackTrace();
			} catch (ConnectionException e)
			{
				e.printStackTrace();
			}

		} catch (MissingParameterException e)
		{
			e.printStackTrace();
		}

		return con;
	}

	public static void main(String[] args)
	{
		DaVKomTSFAbschnitte.getInstance(getDaVConnection());
	}
}
