/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CopyOnWriteArrayList;

import org.omg.CORBA._IDLTypeStub;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFAbschnittBefehl.WrapperListenerBenachrichtigung;

/**
 * Informiert Dialoge/Views ber die von {@link DaVKomTSFSteuerung} gelieferten
 * Informationen.<br>
 * <br>
 * Mit den Methoden
 * {@link GUIModelTSFSteuerung#addListener(SystemObject, PropertyChangeListener)
 * und GUIModelTSFSteuerung#removeListener(SystemObject,
 * PropertyChangeListener)} knnen Listener an-/abgemeldet werden, die ber
 * bestimmte Abschnitte von der Steuerung informiert werden.<br>
 * Mit den Methoden
 * {@link GUIModelTSFSteuerung#addListenerAllgemein(PropertyChangeListener)} und
 * {@link GUIModelTSFSteuerung#removeListenerAllgemein(PropertyChangeListener)}
 * knnen Listener an-/abgemeldet werden, die ber alle Abschnitte informiert
 * werden.<br>
 * Die Nachfolgenden Events werden an dieser Listener bermittelt:
 * 
 * <ul>
 * <li>{@link GUIModelTSFSteuerung#PROP_INIT} zur initialen bermittlung aller
 * {@link ZustandSammlung} fr alle TSF-Abschnitte.</li>
 * <li>{@link GUIModelTSFSteuerung#PROP_ADD} wenn eine {@link ZustandSammlung}
 * fr einen TSF-Abschnitt hinzugefgt oder gendert wurde.</li>
 * <li>{@link GUIModelTSFSteuerung#PROP_SAMMLUNG_REMOVED} wenn eine
 * {@link ZustandSammlung} entfernt wurde.</li>
 * <li>{@link GUIModelTSFSteuerung#PROP_CLEAR_RESET} wenn die gesammte
 * Datenstruktur des Modells zurckgesetzt wurde.</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class GUIModelTSFSteuerung implements PropertyChangeListener,
		IListenerZustandsSammlungReset
{
	public static final String PROP_INIT = "INIT";
	public static final String PROP_ADD = "ADD";
	public static final String PROP_CLEAR_RESET = "CLEAR_RESET";

	/**
	 * Dieser Event wird gesendet, wenn eine Sammlung entfernt wurde.
	 */
	public static final String PROP_SAMMLUNG_REMOVED = "SAMMLUNG_REMOVED";

	/**
	 * Fr Listener, die ber alles informiert werden mchten.
	 */
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * Key = SystemObjekt des jeweiligen Abschnitts (<code>typ.tsfAbschnitt</code>
	 * ), Value=Lister der {@link PropertyChangeListener}.
	 */
	private HashMap<SystemObject, LinkedList<PropertyChangeListener>> _mapListener = new HashMap<SystemObject, LinkedList<PropertyChangeListener>>();

	/**
	 * Key = {@link SystemObject} des jeweiligen Abschnitts (
	 * <code>typ.tsfAbschnitt</code>), Value=Sammlung der Zustnde bis zum
	 * nchsten Reset.
	 */
	private HashMap<SystemObject, ZustandSammlung> _mapSammlungen = new HashMap<SystemObject, ZustandSammlung>();

	/**
	 * Aspekt, auf den gehrt werden soll beim Datenempfang.
	 */
	private String _pidAsp = null;

	/**
	 * Liste der Listener die darber informiert werden mchten, wenn die
	 * ZustandsSammlung zurckgesetzt wird.
	 */
	private CopyOnWriteArrayList<IListenerZustandsSammlungReset> _listenerZustandssammlungReset = new CopyOnWriteArrayList<IListenerZustandsSammlungReset>();

	/**
	 * 
	 * @param pidAsp
	 *          Aspekt, auf dem beim "Empfang" gehrt werden soll.
	 */
	public GUIModelTSFSteuerung(String pidAsp)
	{
		this._pidAsp = pidAsp;
		listenerAnmelden();
	}

	/**
	 * Liefert den Aspekt, auf dem beim "Empfang" gehrt werden soll.
	 * 
	 * @return
	 */
	private String getPidAsp()
	{
		return _pidAsp;
	}

	private void listenerAnmelden()
	{
		DaVKomTSFAbschnittBefehl.getInstance().getModel().addListener(this);
	}

	private void listenerAbmelden()
	{
		DaVKomTSFAbschnittBefehl.getInstance().getModel().removeListener(this);
	}

	protected void putZustand(I_TSFZustand zustand)
	{
		if (zustand.getSo() != null)
		{
			// TODO JH TEST
			// AggregationAbschnittZustand.getInstance().setZustandSteuerung(getPidAsp(),
			// zustandSteuerung);

			ZustandSammlung zustandSteuerungSammlung = null;

			zustandSteuerungSammlung = getZustandSteuerungSammlung(zustand.getSo(),
					true);

			if (zustandSteuerungSammlung != null)
			{
				zustandSteuerungSammlung.putZustandSteuerung(zustand);

				// Listener benachrichtigen
				PropertyChangeListener[] aListener = getListener(zustand.getSo());
				if (aListener.length > 0)
				{
					PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_ADD,
							null, zustandSteuerungSammlung);
					for (PropertyChangeListener pcl : aListener)
					{
						pcl.propertyChange(pce);
					}
				}

				// Allg. Listener benachrichtigen
				_pcs.firePropertyChange(PROP_ADD, null, zustandSteuerungSammlung);
			}

		} else
		{
			System.err.println("Das SystemObjekt von ZustandSteuerung ist null => "
					+ zustand.toString());
		}
	}

	/**
	 * Hier wird darauf gewartet, dass ein bestimmter Zustand erreicht wird, bevor
	 * eine {@link ZustandSammlung} zurckgesetzt werden kann. Die Liste der
	 * {@link M_TSFZustandSteuerung} der {@link ZustandSammlung} wird
	 * zurckgesetzt, d.h. alle Elemente - bis auf das letzte Element - werden
	 * gelscht.
	 * 
	 * @param so
	 */
	private void resetZustandSteuerung(SystemObject so)
	{
		ZustandSammlung sammlung = null;
		synchronized (_mapSammlungen)
		{
			sammlung = _mapSammlungen.get(so);
		}

		if (sammlung != null)
		{
			// sammlung.resetListeZustaende();
			sammlung.resetListeZustaende();

			/*
			 * Listener benachrichtigen.
			 */
			PropertyChangeListener[] aListener = getListener(so);
			if (aListener.length > 0)
			{
				PropertyChangeEvent pce = new PropertyChangeEvent(this,
						PROP_CLEAR_RESET, sammlung, null);
				for (PropertyChangeListener pcl : aListener)
				{
					pcl.propertyChange(pce);
				}
			}

			/*
			 * Allgm. Listener benachrichtigen
			 */
			_pcs.firePropertyChange(PROP_CLEAR_RESET, sammlung, null);
		}
	}

	private void putTSFAbschnittBefehl(M_TSFAbschnittBefehl befehl)
	{
		if ((befehl != null) && (befehl.getSo() != null))
		{
			// TODO JH TEST
			// AggregationAbschnittZustand.getInstance().setAbschnittBefehl(getPidAsp(),
			// befehl);

			SystemObject soAbs = befehl.getSo();
			E_TSFBefehl aktion = befehl.getAktion();
			E_TSFProgrammZustand programmZustand = befehl.getZustand();

			ZustandSammlung zustandSteuerungSammlung = getZustandSteuerungSammlung(
					soAbs, true);

			/*
			 * (NEU!) Reset der Listendarstellung durchfhren wenn ein Wechsel von
			 * Geschaltet in einen anderen Zustand erfolgt.
			 */
			// E_TSFProgrammZustand programmZustandBisher = zustandSteuerungSammlung
			// .getProgrammZustand();
			// if ((programmZustandBisher == E_TSFProgrammZustand.Geschaltet)
			// && (programmZustand != programmZustandBisher))
			// {
			// resetZustandSteuerung(soAbs);
			// }

			// if (befehl.getZustand() != null)
			// {
			//
			// if (!Defs.istBestimmtesProgrammFuerAbschnittErreicht(soAbs.getPid(),
			// befehl.getAktion()))
			// {
			// // resetZustandSteuerung();
			// resetZustandSteuerung(soAbs);
			// }
			// }

			if (aktion != null)
			{
				zustandSteuerungSammlung.setZielProgramm(aktion);
				// listenerBenachrichtigen = true;
			}

			if (programmZustand != null)
			{
				zustandSteuerungSammlung.setProgrammZustand(programmZustand);
			}

			// Spez. Listener
			PropertyChangeListener[] listener = getListener(soAbs);
			if (listener.length > 0)
			{
				PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_ADD, null,
						zustandSteuerungSammlung);
				for (PropertyChangeListener pcl : listener)
				{
					pcl.propertyChange(pce);
				}
			}

			// Allg. Listener
			_pcs.firePropertyChange(PROP_ADD, null, zustandSteuerungSammlung);

			// /*
			// * Reset auf die alten Zustnde durchfhren.
			// */
			// if (programmZustand == E_TSFProgrammZustand.Geschaltet)
			// {
			// resetZustandSteuerung(soAbs);
			// }

			// boolean pruefeObZielPrgErreicht = Defs.pruefeObZielPrgErreicht(soAbs
			// .getPid());
			// System.out.println("Zielprg. Erreicht (Aspekt=\"" + getPidAsp()
			// + "\") => " + pruefeObZielPrgErreicht);
		}
	}

	/**
	 * Liefert eine {@link ZustandSammlung} fr einen Abschnitt.
	 * 
	 * @param pidAbs
	 *          PID des Objekts des Abschnitts vom Typ
	 *          <code>typ.tsfAbschnitt</code>.
	 * @return Liefert die {@link ZustandSammlung} fr den jeweiligen Abschnitt
	 *         oder <code>null</code> wenn nicht verfgbar.
	 */
	public ZustandSammlung getZustandSteuerungSammlung(String pidAbs)
	{
		ZustandSammlung re = null;

		ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
		if (con != null)
		{
			SystemObject soAbs = con.getDataModel().getObject(pidAbs);
			if (soAbs != null)
			{
				re = getZustandSteuerungSammlung(soAbs);
			}
		}

		return re;
	}

	/**
	 * Liefert eine {@link ZustandSammlung} fr einen Abschnitt.
	 * 
	 * @param soAbs
	 *          {@link SystemObject} vom Typ <code>typ.tsfAbschnitt.</code>
	 * @return Kann <code>null</code> sein wenn nicht verfgbar.
	 */
	public ZustandSammlung getZustandSteuerungSammlung(SystemObject soAbs)
	{
		ZustandSammlung re = null;

		if (soAbs != null)
		{
			re = getZustandSteuerungSammlung(soAbs, true);
		}

		return re;
	}

	/**
	 * 
	 * @param soAbs
	 *          SystemObjekt vom Typ <code>typ.tfsAbschnitt</code>.
	 * @param neueSammlungAnlegenWennNichtVorhanden
	 * @return
	 */
	private ZustandSammlung getZustandSteuerungSammlung(SystemObject soAbs,
			boolean neueSammlungAnlegenWennNichtVorhanden)
	{
		ZustandSammlung re = null;

		if (soAbs != null)
		{
			synchronized (_mapSammlungen)
			{
				if (neueSammlungAnlegenWennNichtVorhanden)
				{
					if (!_mapSammlungen.containsKey(soAbs))
					{
						_mapSammlungen.put(soAbs, new ZustandSammlung(soAbs));
					}
				}

				re = _mapSammlungen.get(soAbs);
			}
		}

		return re;
	}

	public void addListener(SystemObject so, PropertyChangeListener pcl)
	{
		synchronized (_mapListener)
		{
			boolean listenerBenachrichtigen = true;

			if (!_mapListener.containsKey(so))
			{
				_mapListener.put(so, new LinkedList<PropertyChangeListener>());
			}

			LinkedList<PropertyChangeListener> lListener = _mapListener.get(so);
			if (!lListener.contains(pcl))
			{
				lListener.add(pcl);
				listenerBenachrichtigen = true;
			}

			if (listenerBenachrichtigen)
			{
				ZustandSammlung zustandSteuerungSammlung = null;
				synchronized (_mapSammlungen)
				{
					zustandSteuerungSammlung = _mapSammlungen.get(so);
				}

				pcl.propertyChange(new PropertyChangeEvent(this, PROP_INIT, null,
						zustandSteuerungSammlung));
			}
		}
	}

	public void removeListener(SystemObject so, PropertyChangeListener pcl)
	{
		synchronized (_mapListener)
		{
			LinkedList<PropertyChangeListener> lListener = _mapListener.get(so);
			if (lListener != null)
			{
				lListener.remove(pcl);
				if (lListener.size() == 0)
				{
					_mapListener.remove(so);
				}
			}
		}
	}

	/**
	 * Liefert die Listener, welche auf die Zustnde eines bestimmten Abschnittes
	 * hren.
	 * 
	 * @param so
	 *          {@link SystemObject} vom Typ <code>typ.tsfAbschnitt</code>.
	 */
	private PropertyChangeListener[] getListener(SystemObject so)
	{
		LinkedList<PropertyChangeListener> re = new LinkedList<PropertyChangeListener>();

		synchronized (_mapListener)
		{
			LinkedList<PropertyChangeListener> lListener = _mapListener.get(so);
			if ((lListener != null) && (lListener.size() > 0))
			{
				for (PropertyChangeListener pcl : lListener)
				{
					re.add(pcl);
				}
			}
		}

		return re.toArray(new PropertyChangeListener[re.size()]);
	}

	public void addListenerAllgemein(PropertyChangeListener listener)
	{
		if (listener != null)
		{
			_pcs.addPropertyChangeListener(listener);

			LinkedList<ZustandSammlung> liste = new LinkedList<ZustandSammlung>();
			synchronized (_mapSammlungen)
			{
				Collection<ZustandSammlung> values = _mapSammlungen.values();
				liste.addAll(values);
			}

			ZustandSammlung[] aSammlung = liste.toArray(new ZustandSammlung[liste
					.size()]);
			listener.propertyChange(new PropertyChangeEvent(this, PROP_INIT, null,
					aSammlung));
		}
	}

	public void removeListenerAllgemein(PropertyChangeListener listener)
	{
		_pcs.removePropertyChangeListener(listener);
	}

	public class ZustandSammlung
	{
		/**
		 * SytemObjekt vom Typ <code>typ.tsfAbschnitt</code>.
		 */
		private SystemObject _soAbs = null;

		/**
		 * Liste der Zustnde der Steuerung.
		 */
		private CopyOnWriteArrayList<I_TSFZustand> _listeZustaende = new CopyOnWriteArrayList<I_TSFZustand>();

		/**
		 * Zeigt an, ob der durchlauf der Steuerung beendet ist oder nicht.
		 */
		private boolean _resetEnde = false;

		/**
		 * Das Ziel-Programm, welches geschaltet werden soll oder wurde.
		 */
		private E_TSFBefehl _zielProgramm = null;

		/**
		 * Gibt auskunft darber, ob das betreffende Programm geschaltet wurde oder
		 * aktuell geschaltet wird.
		 */
		private E_TSFProgrammZustand _programmZustand = null;

		private TimerTask _task;

		/**
		 * 
		 * @param soAbs
		 *          SystemObjekt vom Typ <code>typ.tfsAbschnitt</code>.
		 */
		public ZustandSammlung(SystemObject soAbs)
		{
			this._soAbs = soAbs;
		}

		/**
		 * 
		 * @return SystemObjekt vom Typ <code>typ.tfsAbschnitt</code>.
		 */
		public SystemObject getSoTfsAbschnitt()
		{
			return _soAbs;
		}

		private void putZustandSteuerung(I_TSFZustand zustandSteuerung)
		{
			if (zustandSteuerung != null)
			{
				{
					_listeZustaende.add(zustandSteuerung);

//					/*
//					 * Initial kann der Fall auftreten, dass in der Steuerung viele
//					 * Zustnde gespeichert sind. Dies wieder die View bersicht Steuerung
//					 * vollmllen und ist von uns so nicht gewollt. Leider haben wir nicht
//					 * die Mglichkeit festzustellen, wann konkret der Neustart statt
//					 * gefunden hat, somit ist dies die Quick&Dirty Lsung zum Initialen
//					 * ausmllen.
//					 */
//					if (_task == null)
//					{
//						_task = new TimerTask()
//						{
//
//							@Override
//							public void run()
//							{
//								resetListeZustaendeNeu();
//							}
//						};
//
//						Timer timer = new Timer("Timer Reset ZustandSammlung fr Abschnitt "
//								+ _soAbs.toString());
//						timer.schedule(_task, 2000l);
//					}
				}
			} else
			{
				System.err
						.println(this.getClass().getSimpleName()
								+ "::putZustandSteuerung -> Der bergebene ZustandSteuerung ist null und kann nicht zu einer Sammlung hinzugwefgt werden.");
			}
		}

		private void removeZustandSteuerung(I_TSFZustand zustandSteuerung)
		{
			synchronized (zustandSteuerung)
			{
				_listeZustaende.remove(zustandSteuerung);
			}
		}

		public I_TSFZustand[] getZustaendeSteuerung()
		{
			I_TSFZustand[] re = null;

			synchronized (_listeZustaende)
			{
				re = _listeZustaende.toArray(new I_TSFZustand[_listeZustaende.size()]);
			}

			if (re == null)
			{
				re = new I_TSFZustand[] {};
			}

			return re;
		}

		/**
		 * Liefert das Zielprogramm. Dieses Programm ist geschaltet, wenn der
		 * Programmzustand <code>Geschaltet</code> ist.
		 * 
		 * @return
		 */
		public E_TSFBefehl getZielProgramm()
		{
			return _zielProgramm;
		}

		public void setZielProgramm(E_TSFBefehl zielProgramm)
		{
			_zielProgramm = zielProgramm;
		}

		/**
		 * Liefert den aktuellen Programm / Verarbeitungszustand.
		 * 
		 * @return
		 */
		public E_TSFProgrammZustand getProgrammZustand()
		{
			return _programmZustand;
		}

		public void setProgrammZustand(E_TSFProgrammZustand programmZustand)
		{

			E_TSFProgrammZustand ALT = this._programmZustand;
			this._programmZustand = programmZustand;
			E_TSFProgrammZustand NEU = this._programmZustand;

			/*
			 * Vergleich zwischen altem und neuem Programmzustand. Wenn ein Wechsel
			 * weg von Geschalt statt findet, wird die Liste zurckgesetzt. Listener
			 * werden ber den Reset der Liste benachrichtigt.
			 */
			if ((ALT != NEU) && (NEU != E_TSFProgrammZustand.Geschaltet))
			{
				resetListeZustaendeNeu();
			}

		}

		private void resetListeZustaendeNeu()
		{
//			if (_listeZustaende.size() > 1)
//			{
//				I_TSFZustand letzerZustand = _listeZustaende
//						.get(_listeZustaende.size() - 1);
//				_listeZustaende.clear();
//				_listeZustaende.add(letzerZustand);
//			}
			
			if(_listeZustaende.size() >0)
			{
				_listeZustaende.clear();
			}

			benachrichtigeListenerResetZustandsSammlung(getSoTfsAbschnitt());
		}

		/**
		 * Setzt die Liste auf den letzten Wechsel der Stufe (X>0) -> 0 zurck.
		 */
		private void resetListeZustaende()
		{
			synchronized (_listeZustaende)
			{
				if (_listeZustaende.size() > 0)
				{
					/*
					 * Von hinten nach vorne letzten Index mit 0 finden
					 */
					int idxBisZuDemZuLoeschenIst = -1;

					for (int i = _listeZustaende.size() - 1; i > 0; i--)
					{
						I_TSFZustand iTSFZustandAKT = _listeZustaende.get(i);

						if (iTSFZustandAKT instanceof M_TSFZustandSteuerung)
						{
							M_TSFZustandSteuerung akt = (M_TSFZustandSteuerung) iTSFZustandAKT;

							if ((i - 1) >= 0)
							{
								I_TSFZustand iTSFZustandVor = _listeZustaende.get(i - 1);

								if (iTSFZustandVor instanceof M_TSFZustandSteuerung)
								{
									M_TSFZustandSteuerung vor = (M_TSFZustandSteuerung) iTSFZustandVor;

									if ((akt != null) && (vor != null))
									{
										if ((akt.getStufe() == 0l)
												&& (vor.getStufe() > akt.getStufe()))
										{
											idxBisZuDemZuLoeschenIst = i;
											break;
										}
									}
								}
							}
						}
					}

					if (idxBisZuDemZuLoeschenIst > -1)
					{
						I_TSFZustand[] array = _listeZustaende
								.toArray(new I_TSFZustand[_listeZustaende.size()]);
						_listeZustaende.clear();
						for (int i = idxBisZuDemZuLoeschenIst; i < array.length; i++)
						{
							_listeZustaende.add(array[i]);
						}
					}
				}
			}
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			sb.append("ZuStSam: Abschnitt=");
			if (getSoTfsAbschnitt() != null)
			{
				sb.append(getSoTfsAbschnitt().getPidOrNameOrId());
			} else
			{
				sb.append("-");
			}

			sb.append(", ZielPrg=");
			sb.append(getZielProgramm());
			sb.append(", PrgZust=");
			sb.append(getProgrammZustand());

			return sb.toString();
		}
	}

	protected void dispose()
	{
		listenerAbmelden();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{

		if (evt.getPropertyName().equals(GUIModelTSFAbschnittBefehl.PROP_INIT)
				&& (evt.getNewValue() != null)
				&& (evt.getNewValue() instanceof WrapperListenerBenachrichtigung[]))
		{
			WrapperListenerBenachrichtigung[] aWrapperListenerBenachrichtigungen = (WrapperListenerBenachrichtigung[]) evt
					.getNewValue();
			for (WrapperListenerBenachrichtigung wrapperListenerBenachrichtigung : aWrapperListenerBenachrichtigungen)
			{
				if (wrapperListenerBenachrichtigung.getPidAsp().equals(getPidAsp()))
				{
					if (wrapperListenerBenachrichtigung.getAbschnittBefehl() != null)
					{
						putTSFAbschnittBefehl(wrapperListenerBenachrichtigung
								.getAbschnittBefehl());
					}
				}
			}

		} else if ((evt.getPropertyName()
				.equals(GUIModelTSFAbschnittBefehl.PROP_ADDED))
				&& (evt.getNewValue() != null)
				&& (evt.getNewValue() instanceof WrapperListenerBenachrichtigung))
		{
			WrapperListenerBenachrichtigung wrapperListenerBenachrichtigung = (WrapperListenerBenachrichtigung) evt
					.getNewValue();
			if (wrapperListenerBenachrichtigung.getPidAsp().equals(getPidAsp()))
			{
				if (wrapperListenerBenachrichtigung.getAbschnittBefehl() != null)
				{
					putTSFAbschnittBefehl(wrapperListenerBenachrichtigung
							.getAbschnittBefehl());
				}
			}

		} else if ((evt.getPropertyName()
				.equals(GUIModelTSFAbschnittBefehl.PROP_REMOVED))
				&& (evt.getOldValue() != null)
				&& (evt.getOldValue() instanceof WrapperListenerBenachrichtigung))
		{

			WrapperListenerBenachrichtigung wrapperListenerBenachrichtigung = (WrapperListenerBenachrichtigung) evt
					.getOldValue();
			if (wrapperListenerBenachrichtigung.getPidAsp().equals(getPidAsp()))
			{
				// TODO Lschen, Listener werden nicht benachrichtigt
				if (wrapperListenerBenachrichtigung.getAbschnittBefehl() != null)
				{
					M_TSFAbschnittBefehl abschnittBefehl = wrapperListenerBenachrichtigung
							.getAbschnittBefehl();
					SystemObject soAbschnitt = abschnittBefehl.getSo();
					removeSammlung(soAbschnitt);
				}
			}
		}
	}

	/**
	 * Sammlung entfernen, Listener werden z.Zt. nicht explizit via Listener
	 * darber benachrichtigt.
	 * 
	 * @param soAbschnitt
	 *          {@link SystemObject} des TSFAbschnitts, der aus der Sammlung
	 *          entfernt werden soll.
	 */
	private void removeSammlung(SystemObject soAbschnitt)
	{
		if ((soAbschnitt != null) && (_mapSammlungen != null))
		{

			synchronized (_mapSammlungen)
			{
				ZustandSammlung removedZustandSteuerungSammlung = null;
				if (_mapSammlungen.containsKey(soAbschnitt))
				{
					/*
					 * Wenn der Return-Wert der nachfolgenden Fkt. nicht null ist, dann
					 * wurde eine Sammlung erfolgreich entfernt.
					 */
					removedZustandSteuerungSammlung = _mapSammlungen.remove(soAbschnitt);
				}

				if ((removedZustandSteuerungSammlung != null)
						&& (removedZustandSteuerungSammlung.getSoTfsAbschnitt() != null))
				{
					/*
					 * Allgemeine Listener benachrichtigen,.
					 */
					_pcs.firePropertyChange(PROP_SAMMLUNG_REMOVED,
							removedZustandSteuerungSammlung, null);

					// Spez. Listener benachrichtigen
					PropertyChangeListener[] aListener = getListener(removedZustandSteuerungSammlung
							.getSoTfsAbschnitt());
					if (aListener.length > 0)
					{
						PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_ADD,
								null, removedZustandSteuerungSammlung);
						for (PropertyChangeListener pcl : aListener)
						{
							pcl.propertyChange(pce);
						}
					}
				}
			}
		}
	}

	public void addListenerZustandsSammlungReset(
			IListenerZustandsSammlungReset listener)
	{
		if (!_listenerZustandssammlungReset.contains(listener))
		{
			_listenerZustandssammlungReset.add(listener);
		}
	}

	public boolean removeListenerZustandsSammlungReset(
			IListenerZustandsSammlungReset listener)
	{
		return _listenerZustandssammlungReset.remove(listener);
	}

	private IListenerZustandsSammlungReset[] getListenersZustandSammlungReset()
	{
		return _listenerZustandssammlungReset
				.toArray(new IListenerZustandsSammlungReset[_listenerZustandssammlungReset
						.size()]);
	}

	/**
	 * Benachrichtigt alle Listener, dass fr einen bestimmten Abschnitt ein Reset
	 * der gesammelten Zustnde in der {@link ZustandSammlung} statt gefunden hat.
	 * 
	 * @param soTfsAbschnitt
	 */
	private void benachrichtigeListenerResetZustandsSammlung(
			SystemObject soTfsAbschnitt)
	{
		IListenerZustandsSammlungReset[] aListener = getListenersZustandSammlungReset();
		for (IListenerZustandsSammlungReset listener : aListener)
		{
			listener.resetListe(soTfsAbschnitt);
		}
	}

	@Override
	public void resetListe(SystemObject soTfsAbschnitt)
	{
		if (soTfsAbschnitt != null)
		{
			ZustandSammlung zustandSteuerungSammlung = getZustandSteuerungSammlung(soTfsAbschnitt);
			if (zustandSteuerungSammlung != null)
			{
				zustandSteuerungSammlung.resetListeZustaendeNeu();
			}
		}
	}

}
